--[[ !!
- Performance drop
Use MouseEnter, not Hover.
Or use external check and call only on unique values.
--]]



Text = {};
Text.Step = {
   Height = 1;
   Width = 1;
   Maximum = 5000;
};



-- Calculates width or height of the text
local function getSideSize( objScroll, sideName )
   local i = 0;
   local sideSize;
   local stepSizeH = Text.Step.Height;
   local stepSizeW = Text.Step.Width;
   local maxSteps = Text.Step.Maximum;

   while ( objScroll:IsVisible() ) do
      i = i + 1;
      if ( sideName == "H" ) then
         sideSize = stepSizeH * i;
         Text.Label:SetHeight( sideSize );
      elseif ( sideName == "W" ) then
         sideSize = stepSizeW * i;
         Text.Label:SetWidth( sideSize );
      end

      -- Just in case. To prevent infinite loop
      i = i + 1;
      if ( i > maxSteps ) then
         Turbine.Shell.WriteLine( ">> Error. Text.getSideSize (" .. sideName .. "): " .. i .. " steps, possible infinite loop." );
         return nil;
      end
   end

   return sideSize;
end



--[[ Examples:
Calculate width (limit to 1 symbol on the line) and height, return size
   local w, h = Text:getSize( "1234", Turbine.UI.Lotro.Font.Verdana14 );

Limit width to 50, calculate height, return size
   local w, h = Text:getSize( "1234", Turbine.UI.Lotro.Font.Verdana14, 50 );

All text on single line, calculate width and height, return size
!! Doesn't support \n. 
   local w, h = Text:getSize( "1234", Turbine.UI.Lotro.Font.Verdana14, "Single" );
--]]

-- Resets and sets text's values, calculates and returns text's width and height
Text.getSize = function( self, text, font, widthOrSingleLine )
   local label = Turbine.UI.Label();
   self.Label = label;

   -- Scroll. For measuring text's height
   local vScroll = Turbine.UI.Lotro.ScrollBar();
   vScroll:SetParent( label );
   label:SetVerticalScrollBar( vScroll );

   -- Scroll. For measuring text's width
   local hScroll = Turbine.UI.Lotro.ScrollBar();
   hScroll:SetParent( label );
   label:SetHorizontalScrollBar( hScroll );

   local W;
   local H;

   local bWidth = false;
   if ( type( widthOrSingleLine ) == "number" ) then
      bWidth = true;
      W = widthOrSingleLine;
   end

   self.Label:SetSize( 0, 0 );
   if ( bWidth ) then
      self.Label:SetWidth( W );
   end

   self.Label:SetMultiline( true );
   if ( widthOrSingleLine and not bWidth ) then
      self.Label:SetMultiline( false );
   end

   self.Label:SetFont( font );
   self.Label:SetText( text );

   -- Calculate width
   function hScroll.VisibleChanged()
      if ( not bWidth ) then
         W = getSideSize( hScroll, "W" );
      end
   end
   hScroll.VisibleChanged();

   -- Calculate height
   function vScroll.VisibleChanged()
      H = getSideSize( vScroll, "H" );
   end
   vScroll.VisibleChanged();

   return W, H;
end
